@@ -1,11 +1,16 @@
 package Test::ListCompareSpecial;
 # Contains test subroutines for distribution with List::Compare
-# As of:  May 21, 2004
+# As of:  August 1, 2004
 require Exporter;
 our @ISA       = qw(Exporter);
-our @EXPORT_OK = qw( ok_capture_error ok_seen_a  ok_seen_h ok_any_h _capture ); 
-our %EXPORT_TAGS = (
-    seen => [ qw( ok_capture_error ok_seen_a  ok_seen_h ok_any_h _capture ) ],
+our @EXPORT_OK = qw(
+    ok_capture_error ok_seen_a  ok_seen_h ok_any_h _capture
+    getseen
+ ); 
+our %EXPORT_TAGS = ( seen => [ qw( 
+    ok_capture_error ok_seen_a  ok_seen_h ok_any_h _capture 
+    getseen
+    ) ], 
 );
 
 sub ok_capture_error {
@@ -52,25 +57,16 @@ sub ok_any_h {
 
 sub _capture { my $str = $_[0]; }
 
-1;
-
-#######################################################################
-
-# Code below from earlier version of Test/ListCompareSpecial
-# deleted once test suite adopted Test::Simple as testing modality
-
-#|our @EXPORT      = qw( ok  $testnum );
-#|$testnum = 1;
-
-#|sub ok {
-#|    my $condition = shift;
-#|    print $condition ? "ok $testnum\n" : "not ok $testnum\n";
-#|    $testnum++;
-#|}
-
-#|sub ok_capture_error {
-#|    my $condition = shift;
-#|    print $condition ? "\nok $testnum  IGNORE PRINTOUT;\n        bad values have been correctly detected during initialization.\n" : "not ok $testnum\n";
-#|    $testnum++;
-#|}
+sub getseen {
+    my $allref = shift;
+    my @arr = @$allref;
+    my (@seen);
+    for (my $i=0; $i<=$#arr; $i++) {
+        foreach my $j (@{$arr[$i]}) {
+            $seen[$i]{$j}++;
+        }
+    }
+#    return \@seen;
+    return @seen;
+}
 
